// C Source File
// Created 31/05/2003; 21:54:31

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files
#include <system.h>

#define ITEXT_VERSION "v1.2"
#define ITEXT_RELEASE_DATE "(14/09/2003)"
#define ITEXT_AUTHOR "Emeric Poupon (aka ITM)"

#define MAX_FILE_NAME_LENGTH	32
#define NB_MAX_DIRECTORIES 		40
#define NB_MAX_FILES			50  //pour chaque repertoire!!
#define BUFFER_LENGTH			64

#define MAX_STRING_LEN 24			//pour LA RECHERCHE

#define MEDIUM_FONT	F_6x8
#define SMALL_FONT	F_4x6


#define SCREEN_W (ScrRect->xy.x1+1)
#define SCREEN_H (ScrRect->xy.y1+1)

#define DIAMOND_CHAR_CODE 127
#define NEW_LINE_CODE 13

#define VARIABLE_OFF_X -1

#define OVERVIEW_LINES	3 //2 lignes d'aperu !

#define NB_MAX_PAGES 50

#define true	1
#define false	0
#define NOT_DEFINED -1

typedef struct {
	short nb_pages;
	int page_index[NB_MAX_PAGES]; //Un tableau d'index de pages pour UP+DOWN
	int total_length;
	int current_page;
	int current_offset_y; //en pixel, par rapport au dbut
} FILE_EDIT_INFO;


typedef struct {
	short font;		//la police  utiliser
	short off_x;	//le dplacement en abscisse pour chaque lettre
	short off_y;	//le dplacement en ordonne (pour "centrer" les chiffers sur les cases)
	short off_x_approx;
} ITEXT_FONT;



#define WORD_WRAP		0x00000001
#define CASE_SENSITIVE	0x00000002
#define JUSTIFICATION	0x00000004


//Des que ma flegme repart, je fous tout a dans une stucture! (comme a je pourrai faire un fichier de sauvegarde des paramtres!)
int ITEXT_OPTIONS;
short index_current_font;
ITEXT_FONT* current_font;	//la police courrante pour le jeu
ITEXT_FONT* m_f;				//La police du menu
short cur_dir;
short cur_file;

//Dessine  l'cran  partir du pointeur courant
void draw_cur_file_pos(FILE* fp, int cur_offset, int start_h, int end_h);

void window(const char* title, const char* content, short font);
void window_help(ITEXT_FONT* font, const char* filename);
FILE_EDIT_INFO set_file_edit_info(FILE* fp, ITEXT_FONT *font);

void ask_for_string(char* str, ITEXT_FONT* font);

//retourne l'offset du mot trouv  partir de l'offset start
int find_word(FILE* fp, char* word, int start_offset) ;
//retourne la page o se trouve l'offset demand!
short set_file_page(FILE_EDIT_INFO* f_i,  int offset);

short get_filename(char* filename, ITEXT_FONT* font);

void draw_file_info(char* filename, FILE_EDIT_INFO* f_i);

//on dfinit nos deux types de police
ITEXT_FONT l_font = {F_8x10,	8,	10, 8};	
ITEXT_FONT m_font = {F_6x8,	6,	8, 6};
ITEXT_FONT s_font = {F_4x6,	VARIABLE_OFF_X,6, 6};	//-1 => variable off_X

//On va faire un pseudo page flipping (sigh a sent le coup foireux)
void* screen_buffer[2];
short current_buffer;

typedef struct {
	int x, y;
} VECTORd;

// Main Function
void _main(void)
{
	char filename[MAX_FILE_NAME_LENGTH];
	char buf[BUFFER_LENGTH];
	char buf_search[BUFFER_LENGTH];
	FILE* fp;
	short key = 0;
	short REFRESH_FLAG = FALSE;
	int offset_search_result;
	int start_h = NOT_DEFINED;
	int end_h = NOT_DEFINED;

	FILE_EDIT_INFO f_info;
	
	ESI argptr;
	int argtype;

	if (TI89)
		m_f = &s_font;
	else
		m_f = &m_font;


 	InitArgPtr (argptr);
	if ((argtype = GetArgType (argptr)) == END_TAG || argtype != STR_TAG) {
		if (get_filename(filename, &m_font) == FALSE)
			return;
	}
	else 
		strcpy(filename, GetStrnArg (argptr));
  
	fp = fopen(filename, "r");
	if (!fp) {
		sprintf(buf, "\"%s\" does not exist", filename);
		window("Opening error", buf, m_f->font);
		return;
	}
	
	current_font = &m_font;
	f_info = set_file_edit_info(fp, current_font) ;


	ClrScr();

	//les paramtres par dfaut:
	ITEXT_OPTIONS = 0;
	ITEXT_OPTIONS |= WORD_WRAP;
	ITEXT_OPTIONS |= CASE_SENSITIVE;


	//dessine dessus
	draw_cur_file_pos(fp, f_info.page_index[f_info.current_page], start_h, end_h);

	//Les infos en bas...
	draw_file_info(filename, &f_info);



	while (1) {
		fseek(fp, f_info.page_index[f_info.current_page], SEEK_SET);

		if (REFRESH_FLAG == TRUE) {

			//les infos en bas de l'image des fois que ce soit updat:
			FontSetSys (s_font.font);
			sprintf(buf, "Page %d/%d    ", f_info.current_page+1, f_info.nb_pages);
			DrawStr(0, SCREEN_H+1, buf, A_REPLACE);

			draw_cur_file_pos(fp, f_info.page_index[f_info.current_page], start_h, end_h);	
			
			//et maintenant, on alterne!			
			if (current_buffer == 0)
				current_buffer = 1;
			else 
				current_buffer = 0;

			PortRestore();

		}

		
		REFRESH_FLAG = false;
		key = ngetchx();
		if (key == KEY_ESC) { //esc
			break;
		}
		else if	(key == KEY_APPS) { //APPS
			window_help(m_f, "Help window");
		}
		else if (key == KEY_MODE) { //Mode
				window ("IText "ITEXT_VERSION" "ITEXT_RELEASE_DATE, "By "ITEXT_AUTHOR, m_f->font);
		}
		else if (key == KEY_F1) {//F1
				if (current_font == &s_font)
					continue;
				REFRESH_FLAG = TRUE;
				start_h = end_h = NOT_DEFINED;
				current_font = &s_font;					
				f_info = set_file_edit_info(fp, current_font) ;
				f_info.current_page = 0;
		}
		else if (key == KEY_F2) { //F2
				if (current_font == &m_font)
					continue;
				REFRESH_FLAG = TRUE;
				start_h = end_h = NOT_DEFINED;						
				current_font = &m_font;
				f_info = set_file_edit_info(fp, current_font) ;
				f_info.current_page = 0;
		}
		else if (key == KEY_F3) { //F3
				if (current_font == &l_font)
					continue;
				REFRESH_FLAG = TRUE;					
				start_h = end_h = NOT_DEFINED;		
				current_font = &l_font; 
				f_info = set_file_edit_info(fp, current_font) ;
				f_info.current_page = 0;
		}
		else if (key == KEY_F4) {
				REFRESH_FLAG = false;
				off();			
		}
		else if (key == KEY_F5) {
			ask_for_string(buf_search, m_f);
			if (!buf_search)
				continue;
			if (start_h == NOT_DEFINED && end_h == NOT_DEFINED)
				offset_search_result = find_word(fp, buf_search, f_info.page_index[f_info.current_page]);
			else
				offset_search_result = find_word(fp, buf_search, end_h);
			if (offset_search_result < 0) {
				sprintf(buf, "\"%s\" not found!", buf_search);
				window("Warning", buf, m_f->font);
			}
			else {
				REFRESH_FLAG = TRUE;					
				f_info.current_page = set_file_page(&f_info, offset_search_result);
				start_h = offset_search_result;
				end_h = offset_search_result + strlen(buf_search);
			}
		}
		else if (key == KEY_F6) {
			if (start_h == NOT_DEFINED && end_h == NOT_DEFINED)
				offset_search_result = find_word(fp, buf_search, f_info.page_index[f_info.current_page]);
			else
				offset_search_result = find_word(fp, buf_search, end_h);
			if (offset_search_result < 0) {
				sprintf(buf, "\"%s\" not found!", buf_search);
				window("Warning", buf, m_f->font);
			}
			else {
				REFRESH_FLAG = TRUE;					
				f_info.current_page = set_file_page(&f_info, offset_search_result);
				start_h = offset_search_result;
				end_h = offset_search_result + strlen(buf_search);
			}
		}
		else if (key == KEY_F7) {
			if (get_filename(filename, &m_font) == FALSE)
				continue;
			fclose(fp);
			fp = fopen(filename, "r");
			if (!fp) {
				sprintf(buf, "\"%s\" does not exist", filename);
				window("Opening error", buf, m_f->font);
				PortRestore();
				free(screen_buffer[0]);
				free(screen_buffer[1]);
			}
			f_info = set_file_edit_info(fp, current_font) ;
			f_info.current_page = 0;
			REFRESH_FLAG = TRUE;

			ClrScr();
			draw_file_info(filename, &f_info);			
		}
		else if ((key == 8530 || key == 16721) && f_info.current_page != 0) { //DIAMOND + HAUT
				REFRESH_FLAG = TRUE;			
				start_h = end_h = NOT_DEFINED;						
				f_info.current_page = 0;
		}
		else if ((key == 8536 || key == 16724) && f_info.current_page != f_info.nb_pages-1) { //DIAMOND + BAS
				REFRESH_FLAG = TRUE;			
				start_h = end_h = NOT_DEFINED;						
				f_info.current_page = f_info.nb_pages-1;
		}
		else if (((key == 4436 && TI89) || key == 4440) && f_info.current_page != f_info.nb_pages-1) { //SECOND + BAS
				REFRESH_FLAG = TRUE;			
				start_h = end_h = NOT_DEFINED;						
				f_info.current_page += 3;
				if (f_info.current_page >= f_info.nb_pages)
					f_info.current_page = f_info.nb_pages-1;
		}
		else if (((key == 4433 && TI89) || key == 4434) && f_info.current_page != 0) { //SECOND + HAUT
				REFRESH_FLAG = TRUE;			
				start_h = end_h = NOT_DEFINED;						
				f_info.current_page -= 3;
				if (f_info.current_page < 0)
					f_info.current_page = 0;
				
		}
		else if (key == KEY_UP && f_info.current_page != 0) { //HAUT
				REFRESH_FLAG = TRUE;			
				start_h = end_h = NOT_DEFINED;						
				f_info.current_page--;
				if (f_info.current_page < 0)
					f_info.current_page = 0;
		}
		else if (key == KEY_DOWN && f_info.current_page != f_info.nb_pages-1) { //BAS
				REFRESH_FLAG = TRUE;			
				start_h = end_h = NOT_DEFINED;						
				f_info.current_page++;
				if (f_info.current_page >= f_info.nb_pages)
					f_info.current_page = f_info.nb_pages-1;
		}
	}
	fclose(fp);
}

#define START_X_STR	4
#define START_Y_STR	4
#define TITLE_HEIGHT 20

void window(const char* title, const char* content, short font) {
	short f = FontGetSys();
	WIN_RECT w_r;
	WINDOW	w;
	w_r.x0 = (SCREEN_W - DrawStrWidth (content, font) - START_X_STR*2)/2;
	if (w_r.x0 < 0)
		w_r.x0 = 0;
	w_r.y0 = 20;
	w_r.x1 = DrawStrWidth (content, font) + w_r.x0 + START_X_STR*2;
	w_r.y1 = w_r.y0 + TITLE_HEIGHT + 2 * START_Y_STR;

  WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER|WF_TITLE, title);
  WinFont(&w,font);
  WinActivate(&w);
  WinStrXY(&w,START_X_STR,START_Y_STR	,(char*)content);
	ngetchx();
	WinClose(&w);
	FontSetSys(f);
}


void draw_cur_file_pos(FILE* fp, int cur_offset, int start_h, int end_h) {
	char c, c2;
	char p_c = NEW_LINE_CODE; //par dfaut, on commence le fichier comme si le caractre prcdebnt tait retour  la ligne
	char buf[3];	
	short c_pos_x;
	short c_pos_y;
	short is_a_new_word = TRUE;  
	short temp_pos;
	short nb_char;
	
	FontSetSys (current_font->font); 
	c_pos_x = c_pos_y = 0;
	//Effacer l'cran
	ScrRectFill ( &(SCR_RECT){{0,0,SCREEN_W, SCREEN_H-2}}, &(SCR_RECT){{0,0,SCREEN_W, SCREEN_H-2}}, A_REVERSE); 
	
	//trs simple, on copie le contenu de l'un sur l'autre en dcalant de diff_y pixels :D
	

	while ((c = getc(fp)) != EOF) {
		++cur_offset;
		if (c == NEW_LINE_CODE) {   //passage  la ligne
			++cur_offset;
			is_a_new_word = TRUE;

			c_pos_x = 0;
			++c_pos_y;
		}
		else if (c == ' ' && p_c != NEW_LINE_CODE && c_pos_x == 0)
			continue;
		else {

			if (ITEXT_OPTIONS & WORD_WRAP) {
				//ici on doit rajouter une passe si c'est un nouveau mot pour voir si y'a moyen de l'afficher ou pas
				if ( c == ' ')
					is_a_new_word = TRUE;
				else if (is_a_new_word == TRUE) {
					//il faut scanner le mot suivant
					nb_char = 0;
				
					//un petit backup de la position dans le fichier		
					temp_pos = cur_offset;
				
					//on compte le nb de caractres qu'il reste jusqu' la fin du mot:
					do {
						c2 = getc(fp);
						nb_char++;
					}	while (c2 != NEW_LINE_CODE && c2 != ' ' && c2 != EOF);
	
					//et maintenant, c'est l qu'on estime si c'est bon ou pas :D
					if (c_pos_x + current_font->off_x_approx*(nb_char) > SCREEN_W) {
						//passage  la ligne
						c_pos_x = 0;
						++c_pos_y;
					}
					is_a_new_word = FALSE;
					fseek(fp, temp_pos, SEEK_SET);
				}
			}
			
			if (cur_offset > start_h && cur_offset <= end_h) 
				DrawChar (c_pos_x, c_pos_y * current_font->off_y, c, A_REVERSE);
			else
				DrawChar (c_pos_x, c_pos_y * current_font->off_y, c, A_NORMAL);
						
			if (current_font->off_x == VARIABLE_OFF_X) {
				sprintf(buf, "%c", c);
				c_pos_x += DrawStrWidth(buf, current_font->font);
			}
			else 
				c_pos_x += current_font->off_x;
							
			if (c_pos_x + current_font->off_x_approx > SCREEN_W) {
				c_pos_x = 0;
				c_pos_y++;
			}
		}
		
		if (c_pos_y + 1 > (SCREEN_H/ current_font->off_y)) //on condamne  la dernire ligne
			break;
		p_c = c;
	}
	return;
}


#define SUBDIV_W	12

void window_help(ITEXT_FONT* font, const char* filename) {
	short f = FontGetSys();
	char buf[BUFFER_LENGTH];
	WIN_RECT w_r;
	WINDOW	w;

	w_r.x0 = SCREEN_W/SUBDIV_W;
	if (w_r.x0 < 0)
		w_r.x0 = 0;
	if (TI89)
		w_r.y0 = 2;	//De 20 -> on va dterminer
	else 
		w_r.y0 = 10;
	w_r.x1 = SCREEN_W*(SUBDIV_W-1)/SUBDIV_W;
	w_r.y1 = font->off_y*10 + START_Y_STR*2 + w_r.y0 + TITLE_HEIGHT; //4 lignes  afficher Le 20 c'est la hauteur de la barre de titre

	
	WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER|WF_TITLE, filename);
	WinFont(&w,font->font);
	WinActivate(&w);
	WinStrXY(&w,START_X_STR,START_Y_STR						,"F1 : Small font");
	WinStrXY(&w,START_X_STR,START_Y_STR+(font->off_y)		,"F2 : Medium font");
	WinStrXY(&w,START_X_STR,START_Y_STR+2*(font->off_y)		,"F3 : Large font");

	WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR					,"F4 : TI -> OFF");	
	WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+(font->off_y)		,"F5 : Find...");
	WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+2*(font->off_y)	,"F6 : Find next");
	

	sprintf(buf, "    %c + UP : First page", DIAMOND_CHAR_CODE);
	WinStrXY(&w,START_X_STR,START_Y_STR+4*(font->off_y)		,buf);
	sprintf(buf, "  %c + DOWN : Last page", DIAMOND_CHAR_CODE);	
	WinStrXY(&w,START_X_STR,START_Y_STR+5*(font->off_y)		,buf);
	WinStrXY(&w,START_X_STR,START_Y_STR+6*(font->off_y)		,"  2nd + UP : Page - 3");
	WinStrXY(&w,START_X_STR,START_Y_STR+7*(font->off_y)		,"2nd + DOWN : Page + 3 ");	

	WinStrXY(&w, START_X_STR,START_Y_STR+9*(font->off_y)		,"Mode : About...");
	WinStrXY(&w, START_X_STR,START_Y_STR+10*(font->off_y)		," Esc : Exit");
	WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+9*(font->off_y)	,"F7 : Open");

	if (TI89)
		WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+10*(font->off_y)	,"89 detected!");
	else if (TI92PLUS)
		WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+10*(font->off_y)	,"92+ detected!");
	else if (V200)
		WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+10*(font->off_y)	,"V200 detected!");
	else
		WinStrXY(&w,(w_r.x1-w_r.x0)/2+START_X_STR,START_Y_STR+10*(font->off_y)	,"Unknown calc!");

	ngetchx();
	
	WinClose(&w);
	FontSetSys(f);
	
}

FILE_EDIT_INFO set_file_edit_info(FILE* fp, ITEXT_FONT *font) {
	FILE_EDIT_INFO f_i;
	char buf[2];
	char c, c2;
	char p_c = NEW_LINE_CODE;
	int char_offset = 0;
	short  c_pos_x;
	short  c_pos_y;
	short is_a_new_word = TRUE;
	short nb_char;
	int temp_pos;
	
	rewind(fp);

	c_pos_x = c_pos_y = 0;
	//la position actuelle du char en train d'tre dessin sur l'cran

	f_i.nb_pages = 1;
	f_i.page_index[0] = 0;
	
		
	while ((c = getc(fp)) != EOF) {
		++char_offset;			
		if (c == NEW_LINE_CODE) {   //passage  la ligne
			//codage sur deux octets???
			++char_offset;	

			is_a_new_word = TRUE;
			c_pos_x = 0;			
			++c_pos_y;
		}
		//le coup (char)(espace)(char), si l'espace tombe en dbut de ligne on l'enlve... 
		else if (c == ' ' && p_c != NEW_LINE_CODE && c_pos_x == 0)
			continue;
		else {

			if (ITEXT_OPTIONS & WORD_WRAP) {
				//ici on doit rajouter une passe si c'est un nouveau mot pour voir si y'a moyen de l'afficher ou pas
				if (c == ' ')
					is_a_new_word = TRUE;
				else if (is_a_new_word == TRUE) {
					//il faut scanner le mot suivant
					nb_char = 0;
				
					//un petit backup de la position dans le fichier		
					temp_pos = char_offset;
				
					//on compte le nb de caractres qu'il reste jusqu' la fin du mot:
					do {
						c2 = getc(fp);
						nb_char++;
					}	while (c2 != NEW_LINE_CODE && c2 != ' ' && c2 != EOF);
	
					//et maintenant, c'est l qu'on estime si c'est bon ou pas :D
					if (c_pos_x + current_font->off_x_approx*(nb_char) > SCREEN_W) {
						//passage  la ligne
						c_pos_x = 0;
						++c_pos_y;
						char_offset--; //si on a prouv que le mot tait  remettre  la ligne, on revient au caractre d'avant de faon  retraiter le mot dans son intgralit					
					}

						
					fseek(fp, temp_pos, SEEK_SET);
						
					is_a_new_word = FALSE;

				}
			}

			if (font->off_x == VARIABLE_OFF_X) {
				sprintf(buf, "%c", c);
				c_pos_x += DrawStrWidth(buf, font->font);
			}
			else 
				c_pos_x += font->off_x;

			if (c_pos_x + font->off_x_approx > SCREEN_W) {
				c_pos_x = 0;
				c_pos_y++;
			}
		}
		
		if (c_pos_y  >= (SCREEN_H / font->off_y)) {
			f_i.page_index[f_i.nb_pages] = char_offset;
			++f_i.nb_pages;
			c_pos_y = 0;
		}
		p_c = c;
	
	}
	rewind(fp);	
	f_i.total_length = char_offset;
	f_i.current_page = 0;
	return f_i;
}

//retourne l'offset du mot trouv  partir de l'offset start
int find_word(FILE* fp, char* word, int start_offset) {
	char c;
	char buf1[BUFFER_LENGTH];
	char buf2[BUFFER_LENGTH];
	int offset = start_offset;
	short index_str = 0;
	short strl = strlen(word);
	fseek(fp, start_offset, SEEK_SET);
	
	while ((c = getc(fp)) != EOF) {
		++offset;
		
		//cas spcial de nouvelle ligne
		if (c == NEW_LINE_CODE) {
			++offset;
			index_str = 0;
			continue;
		}
		
		sprintf(buf1, "%c", c);
		sprintf(buf2, "%c", word[index_str]);
		if (ITEXT_OPTIONS & CASE_SENSITIVE) {
			if (cmpstri(buf2, buf1) == 0) {
				if (index_str == strl-1)
					return offset - strl;
				++index_str;
				continue;
			}
		}
		else {
			if (strcmp(buf2, buf1) == 0) {
				if (index_str == strl-1)
					return offset - strl;
				++index_str;
				continue;
			}
		}

		index_str = 0;
	}
	return -1;
}

//retourne la page o se trouve l'offset demand!
short set_file_page(FILE_EDIT_INFO* f_i,  int offset) {
	short i;
	
	for (i = 0; i < f_i->nb_pages; ++i) {
		if (offset >= f_i->page_index[i] && ( ( i+1 < f_i->nb_pages && offset < f_i->page_index[i+1]) || i+1 >= f_i->nb_pages))
			return i;
	}
	return false;
}



void ask_for_string(char* str, ITEXT_FONT* font) {

	char buf_d[BUFFER_LENGTH];
	int key;
	int i = strlen(str);
	short f = FontGetSys();
	WIN_RECT w_r;
	WINDOW	w;

	w_r.x0 = (SCREEN_W/4 - START_X_STR*2)/2;
	if (w_r.x0 < 0)
		w_r.x0 = 0;
	w_r.y0 = 20;	//De 20 -> on va dterminer
	w_r.x1 = SCREEN_W*3/4 + w_r.x0 + START_X_STR*2;
	w_r.y1 = font->off_y + START_Y_STR + w_r.y0 + TITLE_HEIGHT; //4 lignes  afficher Le 20 c'est la hauteur de la barre de titre

	WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER|WF_TITLE, "Find a string");
	WinFont(&w,font->font);
	WinActivate(&w);
	WinAttr(&w,A_REPLACE);


	do {
		
		sprintf(buf_d, "%s|  ", str);  
		WinStrXY(&w,START_X_STR,START_Y_STR, buf_d);
	
		key = ngetchx();

      if (key == KEY_BACKSPACE && i) //i != 0
        i--;
      else if (key == KEY_ESC) {
			str = NULL;
			break;
      }
      if (((key >= ' ' && key <= '~') || (key >= 224 && key <= 252)) && i < MAX_STRING_LEN)
        str[i++] = key;
      
      str[i] = 0; //on efface le nouveau cran
      
	} while (key != KEY_ENTER);

	WinClose(&w);
	FontSetSys(f);
	return;	
 }

typedef struct {
	char name[NB_MAX_FILES][MAX_FILE_NAME_LENGTH];
	short nb_files;
	short cur_index;
	short size[NB_MAX_FILES];
} VAT_ENTRY;




short get_filename(char* filename, ITEXT_FONT* font) {
 	char buf[BUFFER_LENGTH];

	VAT_ENTRY	file;	//l'entre pour les fichiers
	VAT_ENTRY	dir;	//les repertoires maintenant
	
 	short REFRESH_VARIABLE_NAMES = TRUE;
 	short REFRESH_OVERVIEWED_FILE = TRUE;
 	short i = 0;
 	short key;
 	
 	int NB_OVERVIEW_CHAR_PER_LINE; //pseudo constante ici
 	
 	char c;
 	
 	FILE *fp;			//ca c'est pour l'aperu du fichier... overview
 		
	WIN_RECT w_r;
	WINDOW	w;
	SYM_ENTRY* sym;

	w_r.x0 = (SCREEN_W/4 - START_X_STR*2)/2;
	if (w_r.x0 < 0)
		w_r.x0 = 0;
	w_r.x1 = SCREEN_W*3/4 + w_r.x0 + START_X_STR*2;
	if (w_r.x1 >= SCREEN_W)
		w_r.x1 = SCREEN_W-1;

	w_r.y0 = 20;	//De 20 -> on va dterminer
	w_r.y1 = font->off_y*(3 + OVERVIEW_LINES) + START_Y_STR + w_r.y0 + TITLE_HEIGHT; //4 lignes  afficher Le 20 c'est la hauteur de la barre de titre
	
	NB_OVERVIEW_CHAR_PER_LINE = ((w_r.x1 - w_r.x0 - 2*START_X_STR)/font->off_x);
	
	WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER|WF_TITLE, "IText "ITEXT_VERSION" "ITEXT_RELEASE_DATE);
	WinFont(&w,font->font);
	WinActivate(&w);
	WinAttr(&w,A_REPLACE);


	FolderOp(NULL, FOP_ALL_FOLDERS);
	
	//On commence par construire la liste des repertoires:
	sym = SymFindFirst (NULL, 0);
	while (sym)
	{
		strcpy (dir.name[i++], sym->name);
		sym = SymFindNext ();
	}
	dir.nb_files = i;
	dir.cur_index = 0;

	//Et bien sur, on doit rafraichir la liste des fichiers pour le premier repertoire.
	REFRESH_VARIABLE_NAMES = TRUE;

	//Dans cette boucle, l'utilisateur choisit son repertoire/fichier
	while (1) {
		if (REFRESH_VARIABLE_NAMES == TRUE) {
			file.cur_index = i = 0;
			sym = SymFindFirst (SYMSTR (dir.name[dir.cur_index]), FO_SINGLE_FOLDER);			
			while (sym)
			{
				if (GetArgType(HToESI(sym->handle)) == TEXT_TAG) {
					strcpy (file.name[i], sym->name);
					file.size[i++] = ((MULTI_EXPR*) HeapDeref (sym->handle))->Size + 2;
				}
				sym = SymFindNext ();				
			}
			file.cur_index = 0;
			file.nb_files = i;
			REFRESH_VARIABLE_NAMES = FALSE;
			REFRESH_OVERVIEWED_FILE = TRUE;
		}

		if (REFRESH_OVERVIEWED_FILE == TRUE)
			WinClr(&w);
		
		sprintf(buf, "Directory : %s", dir.name[dir.cur_index]);
		WinStrXY(&w,START_X_STR,START_Y_STR, buf);
		
		if (file.nb_files >=1)
			sprintf(buf, "     File : %s", file.name[file.cur_index]);
		else {
			sprintf(buf, "     File : Empty!");
			file.size[file.cur_index] = 0;
		}
		WinStrXY(&w,START_X_STR,START_Y_STR+font->off_y, buf);
		

		WinFont(&w, s_font.font);

		if (!TI89) {
			sprintf(buf, "[ %c | %c Files]     [ %c | %c Directories ]     Size : %d", 21, 22, 23, 24, file.size[file.cur_index]); //21->24 c'est les fleches, il faut bien enter  la main le caractre hein!
			WinStrXY(&w,START_X_STR,START_Y_STR+font->off_y*2 + 3, buf); //le +3 c'est juste du style...
		}
		else {
			sprintf(buf, "[ %c | %c Files]     [ %c | %c Directories ]", 21, 22, 23, 24); //21->24 c'est les fleches, il faut bien enter  la main le caractre hein!
			WinStrXY(&w,START_X_STR,START_Y_STR+font->off_y*2 + 3, buf); //le +3 c'est juste du style...
		}

		WinFont(&w,font->font);

		if( REFRESH_OVERVIEWED_FILE == TRUE && file.nb_files >= 1) {
			REFRESH_OVERVIEWED_FILE = FALSE;

			sprintf(filename, "%s\\%s", dir.name[dir.cur_index], file.name[file.cur_index]);			
			
		
			fp = fopen(filename, "r");

			//normalement, c'est bon, on est sur que le fichier existe ici...
			for (i = 0; i < NB_OVERVIEW_CHAR_PER_LINE*OVERVIEW_LINES; ++i) {		
				c = getc(fp);
				if (c == EOF)
					break;
				else if (c == NEW_LINE_CODE) {
					while (i++ % NB_OVERVIEW_CHAR_PER_LINE != 0)
						;
					continue;
				}

				WinCharXY (&w, START_X_STR + (i%NB_OVERVIEW_CHAR_PER_LINE)*font->off_x, (4 + i/NB_OVERVIEW_CHAR_PER_LINE) * font->off_y, c, 1); 
			}
			fclose(fp);			
		}

		key = ngetchx();
		if (key == KEY_UP) {
			REFRESH_VARIABLE_NAMES = TRUE;
			REFRESH_OVERVIEWED_FILE = TRUE;
			if (dir.cur_index == 0)
				dir.cur_index = dir.nb_files - 1;
			else
				--(dir.cur_index);

		}		
		if (key == KEY_DOWN) {
			REFRESH_OVERVIEWED_FILE = TRUE;			
			REFRESH_VARIABLE_NAMES = TRUE;
			if (dir.cur_index == dir.nb_files - 1)
				dir.cur_index = 0;
			else
				++(dir.cur_index);
		}
		if (key == KEY_LEFT) {
			REFRESH_OVERVIEWED_FILE = TRUE;
			if (file.cur_index == 0)
				file.cur_index = file.nb_files - 1;
			else
				--(file.cur_index);
		}		
		if (key == KEY_RIGHT) {
			REFRESH_OVERVIEWED_FILE = TRUE;			
			if (file.cur_index == file.nb_files - 1)
				file.cur_index = 0;
			else
				++(file.cur_index);
		}
		if (key == KEY_ENTER && file.nb_files != 0)
			break;
		if (key == KEY_ESC) {
			WinClose(&w);
			return FALSE;
		}
	
	}

	WinClose(&w);


	FolderOp(NULL, FOP_ALL_FOLDERS);

	sprintf(filename, "%s\\%s", dir.name[dir.cur_index], file.name[file.cur_index]);
	return TRUE;
}

void draw_file_info(char* filename, FILE_EDIT_INFO* f_i) {
	char buf[BUFFER_LENGTH];
	short f = FontGetSys();
	FontSetSys (s_font.font);
	DrawLine (0, SCREEN_H-1, SCREEN_W, SCREEN_H-1, A_NORMAL);
	if (TI89)
		sprintf(buf, "APPS : Help | %s", filename);
	else
		sprintf(buf, "| 92+ | APPS : Help | %d bytes | %s", f_i->total_length, filename);
	DrawStr(SCREEN_W-DrawStrWidth(buf, s_font.font)-1, SCREEN_H+1, buf, A_NORMAL);			

	FontSetSys (s_font.font);
	sprintf(buf, "Page %d/%d    ", f_i->current_page+1, f_i->nb_pages);
	DrawStr(0, SCREEN_H+1, buf, A_REPLACE);
	
	FontSetSys(f);
}
